#include <SDL.h>
#include <stdlib.h>

#include "def.h"
#include "log.h"
#include "log.h"
#include "physics.h"
#include "opengl.h"
#include "ship.h"
#include "pilot.h"
#include "player.h"
#include "joystick.h"
#include "pilot.h"

static int quit = 0;

static unsigned int time = 0;

// Prototypes.
static void handle_keydown(SDLKey key);
static void handle_keyup(SDLKey key);

// Update.
static void update_all(void);

int main(int argc, const char** argv) {
  SDL_Event event;

  // Default window params.
  gl_screen.w = 800;
  gl_screen.h = 640;
  gl_screen.fullscreen = 0;
  if(gl_init()) {
    // Initializes video output.
    WARN("Error initializing video output, exiting...");
    exit(EXIT_FAILURE);
  }

  // Input.
  if(joystick_init())
    WARN("Error initializing joystick input");

  // Data loading.
  ships_load();

  // Testing.
  unsigned int player_id;
  player_id = pilot_create(get_ship("Ship"), "Player", NULL, NULL, PILOT_PLAYER);
  gl_bindCamera(&get_pilot(player_id)->solid->pos);

  pilot_create(get_ship("Miss. Test"), NULL, NULL, NULL, 0);

  time = SDL_GetTicks();

  // Main looops.
  while(!quit) {
    // Event loop.
    while(SDL_PollEvent(&event)) {
      switch(event.type) {
        case SDL_KEYDOWN:
          handle_keydown(event.key.keysym.sym);
          break;
        case SDL_KEYUP:
          handle_keyup(event.key.keysym.sym);
          break;
        case SDL_QUIT:
          quit = 1;
          break;
      }
    }
    update_all();
  }
  // Unload data.
  pilots_free();
  ships_free();
  gl_exit(); // Kills video output.
  exit(EXIT_SUCCESS);
}

// Handle keydown events.
static void handle_keydown(SDLKey key) {
  switch(key) {
    case SDLK_ESCAPE:
      quit = 1;
      break;
    case SDLK_a:
    case SDLK_LEFT:
      player_setFlag(PLAYER_FLAG_MOV_LEFT);
      break;
    case SDLK_d:
    case SDLK_RIGHT:
      player_setFlag(PLAYER_FLAG_MOV_RIGHT);
      break;
    case SDLK_w:
    case SDLK_UP:
      player_setFlag(PLAYER_FLAG_MOV_ACC);
      break;      
    default:
      break;
  }
}

// Handle keyup events.
static void handle_keyup(SDLKey key) {
  switch(key) {
    case SDLK_a:
    case SDLK_LEFT:
      player_rmFlag(PLAYER_FLAG_MOV_LEFT);
      break;
    case SDLK_d:
    case SDLK_RIGHT:
      player_rmFlag(PLAYER_FLAG_MOV_RIGHT);
      break;
    case SDLK_w:
    case SDLK_UP:
      player_rmFlag(PLAYER_FLAG_MOV_ACC);
      break;
    default:
      break;
  }
}

// Update all the things.
// Pilots:
//  -- Think (ai).
//  -- Solid.
static void update_all(void) {
  FP dt = (FP)(SDL_GetTicks() - time) / 1000.0;
  time = SDL_GetTicks();
  glClear(GL_COLOR_BUFFER_BIT);

  pilots_update(dt);

  SDL_GL_SwapBuffers();
}

